-- Load plugins
-- Bootstrap packer if needed {{
local fn = vim.fn
local install_path = fn.stdpath('data')..'/site/pack/packer/start/packer.nvim'
if fn.empty(fn.glob(install_path)) > 0 then
   PackerBootstrap = fn.system({'git', 'clone', '--depth', '1', 'https://github.com/wbthomason/packer.nvim', install_path})
   vim.cmd 'packadd packer.nvim'
end
-- }}

local use = require('packer').use

return require('packer').startup({function()
   -- TODO: Find a better way to organize this

   -- impatient (needs to go early)
   use { -- lewis6991: impatient:      Make nvim load faster by caching them - needs to load early
      'lewis6991/impatient.nvim',
      config = function()
         require('impatient')
      end,
   }

   -- Packer
   use { -- packer:                    plugin manager
      'wbthomason/packer.nvim',
      config = function()
         -- Map :PS to :PackerSync
         vim.cmd 'command! PS PackerSync'
         require("packer").init({
            autoremove = true,
         })
      end
   }

   -- Colors (decorations)
   use { -- treesitter:                syntax highlighting and more
      'nvim-treesitter/nvim-treesitter', run = ':TSUpdate',
      config = function()
         require('blake.lsp').treesitter()
      end,
   }
   use { -- onedark.nvim:              onedark theme
      'navarasu/onedark.nvim',
      config = function()
         -- vim.g.onedark_transparent_background = true
         vim.cmd [[
            "au ColorScheme onedark hi TabLine gui=none guibg='#282C34' guifg='#5C6370'  " all tabs color
            au ColorScheme onedark hi TabLineSel       guibg='#282C34' guifg='#B5BBC7'  " Highlighted tab color
            "au ColorScheme onedark hi TabLineFill      guibg='#282C34'                  " tabline portion without tabs (right-hand side)
         ]]
      end
   }
   use { -- tokyonight.nvim:           tokyonight theme
      'folke/tokyonight.nvim',
      config = function()
         -- vim.g.onedark_transparent_background = true
         vim.cmd [[
            au ColorScheme tokyonight hi TabLine gui=none guibg='#1D202F' guifg='#565F89'  " all tabs color
            au ColorScheme tokyonight hi TabLineSel       guibg='#24283B' guifg='#C0CAF5'  " Highlighted tab color
            au ColorScheme tokyonight hi TabLineFill      guibg='#1D202F'                  " tabline portion without tabs (right-hand side)
         ]]
         -- Has to be put here so it runs synchronously
         vim.cmd 'colorscheme tokyonight'
      end,
   }
   use { -- newspring:                 Good light theme for writing
      'NLKNguyen/papercolor-theme',
   }
   use { -- statusline:                lualine
      'nvim-lualine/lualine.nvim',
      requires = {'kyazdani42/nvim-web-devicons', opt = true},
      config = function()
         require('blake.other').lualine()
      end,
   }
   use { -- nvim-tabline:              batter looking tabline
      'alvarosevilla95/luatab.nvim',
      requires = 'kyazdani42/nvim-web-devicons',
      config = function()
         require('luatab').setup()
      end
   }
   use { -- nvim-colorizer:            css color tag highlighter (ex- #FFB13B)
      'norcalli/nvim-colorizer.lua',
      config = function()
         vim.cmd 'command! COL ColorizerToggle'
      end,
   }
   use { -- ifdef-highlighting:        highlight c/c++ ifdef's - :Define :Undefine
      'vim-scripts/ifdef-highlighting'
   }

   -- IDE features
   ---- LSP
   use { -- lspinstall:                language server installer, (lspconfig is here too)
      'williamboman/nvim-lsp-installer',
      { -- lspconfig: default language server configurations
         'neovim/nvim-lspconfig',
         config = function()
            require('blake.lsp').lspinstall()
            require('blake.lsp').lspconfig()
         end
      }
   }
   use { -- ts-context-commentstring:  set commentstring based on context from treesitter (two langs in one file)
      'JoosepAlviste/nvim-ts-context-commentstring',
      requires = 'nvim-treesitter/nvim-treesitter',
      config = function()
         require'nvim-treesitter.configs'.setup {
            context_commentstring = {
               enable = true,
               enable_autocmd = false,
            },
         }
      end,
   }
   use { -- cmp:                       completion menu
      'hrsh7th/nvim-cmp',
      config = function()
         require('blake.lsp').cmp()
      end,
      requires = { -- (mostly) nvim-cmp sources (not required by cmp, but reqired by my configuration)
         'onsails/lspkind-nvim', -- icons for entries of the completion menu
         'hrsh7th/cmp-path',
         'hrsh7th/cmp-nvim-lua',
         'hrsh7th/cmp-nvim-lsp',
         'saadparwaiz1/cmp_luasnip',
         'ray-x/cmp-treesitter',
         'hrsh7th/cmp-calc',
         'f3fora/cmp-spell',
         'hrsh7th/cmp-buffer',
         'hrsh7th/cmp-cmdline',
         'hrsh7th/cmp-emoji',
      },
   }
   use { -- luasnip:                   snippits plugin
      'L3MON4D3/LuaSnip',
      config = function()
         require("luasnip.loaders.from_vscode").lazy_load()
      end
   }
   use { -- friendly-snippets:         lots of good snippits
      'rafamadriz/friendly-snippets'
   }
   use { -- lsp_signature:             function parameter previews
      'ray-x/lsp_signature.nvim',
      after = 'nvim-lspconfig',
      config = function()
         require('blake.lsp').signature()
      end,
   }
   use { -- symbols-outline:           treesitter-based document outline (SO)
      'simrat39/symbols-outline.nvim',
      config = function()
         vim.cmd 'command! SO SymbolsOutline'
         vim.cmd 'nnoremap <Leader>so <cmd>SymbolsOutline<CR>'
      end,
   }
   -- use { -- ALE:                    Support for lots of linters, etc
   --    'dense-analysis/ale',
   --    ft = {'sh', 'zsh', 'zshrc', 'bashrc', 'bash'},
   --    config = function()
   --       vim.g.ale_disable_lsp = 1
   --    end
   -- }
   use { -- null-ls:                   Support for lots of programming tools, but through nvim lsp
      'jose-elias-alvarez/null-ls.nvim',
      requires = 'nvim-lua/plenary.nvim',
      config = function()
         local null_ls = require("null-ls")
         require("null-ls").setup({
            sources = {
               null_ls.builtins.code_actions.refactoring,
               null_ls.builtins.formatting.clang_format,
            }
         })
      end
   }
   use { -- refactoring:               Refactoring for neovim
      'ThePrimeagen/refactoring.nvim',
      requires = {
         {"nvim-lua/plenary.nvim"},
         {"nvim-treesitter/nvim-treesitter"}
      },
      config = function()
         require('refactoring').setup({})
         -- Remaps for the refactoring operations currently offered by the plugin
         vim.api.nvim_set_keymap("v", "<leader>re", [[ <Esc><Cmd>lua require('refactoring').refactor('Extract Function')<CR>]], {noremap = true, silent = true, expr = false})
         vim.api.nvim_set_keymap("v", "<leader>rf", [[ <Esc><Cmd>lua require('refactoring').refactor('Extract Function To File')<CR>]], {noremap = true, silent = true, expr = false})
         vim.api.nvim_set_keymap("v", "<leader>rv", [[ <Esc><Cmd>lua require('refactoring').refactor('Extract Variable')<CR>]], {noremap = true, silent = true, expr = false})
         vim.api.nvim_set_keymap("v", "<leader>ri", [[ <Esc><Cmd>lua require('refactoring').refactor('Inline Variable')<CR>]], {noremap = true, silent = true, expr = false})
         -- Extract block doesn't need visual mode
         vim.api.nvim_set_keymap("n", "<leader>rb", [[ <Cmd>lua require('refactoring').refactor('Extract Block')<CR>]], {noremap = true, silent = true, expr = false})
         vim.api.nvim_set_keymap("n", "<leader>rbf", [[ <Cmd>lua require('refactoring').refactor('Extract Block To File')<CR>]], {noremap = true, silent = true, expr = false})
         -- Inline variable can also pick up the identifier currently under the cursor without visual mode
         vim.api.nvim_set_keymap("n", "<leader>ri", [[ <Cmd>lua require('refactoring').refactor('Inline Variable')<CR>]], {noremap = true, silent = true, expr = false})
         -- Refactoring telescope integration
         -- load refactoring Telescope extension
         require("telescope").load_extension("refactoring")
         -- remap to open the Telescope refactoring menu in visual mode
         vim.api.nvim_set_keymap(
            "v",
            "<leader>rr",
            "<Esc><cmd>lua require('telescope').extensions.refactoring.refactors()<CR>",
            { noremap = true }
         )
      end,
   }
   ---- DAP (Debug Adapter Protocol)
   use { -- nvim-dap:                  DAP support
      'mfussenegger/nvim-dap',
      config = function()
         require('blake.dap').nvim_dap()
      end
   }
   -- use { -- dap-buddy:              DAP adapter installer
   --    'Pocco81/dap-buddy.nvim',
   --    config = function()
   --       require('blake.dap').dap_buddy()
   --    end
   -- }
   use { -- virtual-text:              Print Variable names while debugging
      'theHamsta/nvim-dap-virtual-text',
      config = function()
         require("nvim-dap-virtual-text").setup()
      end
   }
   use { -- nvim-dap-ui:               a UI for nvim-dap
      'rcarriga/nvim-dap-ui',
      config = function()
         require('blake.dap').dap_ui()
      end,
   }

   ---- Other IDE features
   use { -- telescope:                 fuzzy finder for finding fuzzy things
      'nvim-telescope/telescope.nvim',
      requires = { {'nvim-lua/plenary.nvim'} },
      config = function()
         require('blake.other').telescope()
      end
   }
   use { -- gitsigns:                  git integration
      'lewis6991/gitsigns.nvim',
      requires = {
         'nvim-lua/plenary.nvim'
      },
      config = function()
         require('blake.other').gitsigns()
      end,
   }
   -- use { -- nvim-tree:              file manager
   --    'kyazdani42/nvim-tree.lua',
   --    requires = 'kyazdani42/nvim-web-devicons',
   --    config = function()
   --       require('blake.other').nvimtree()
   --    end
   -- }
   use { -- toggleterm:                terminal
      'akinsho/toggleterm.nvim',
      branch = 'main',
      config = function()
         require('blake.other').toggleterm()
      end
   }
   use { -- neoscroll:                 Smooth Scrolling
      'karb94/neoscroll.nvim',
      disable = true,
      config = function()
         require('blake.other').neoscroll()
      end,
   }
   use { -- auto-session:              automatic session management
      'rmagatti/auto-session',
      config = function()
         require('blake.other').autosession()
      end
   }
   use { -- glow:                      Markdown preview
      'ellisonleao/glow.nvim',
      ft = { 'md', 'markdown', }
   }
   use { -- zen mode:                  (ZenMode or ZM)
      "folke/zen-mode.nvim",
      config = function()
         require('blake.other').zenmode()
         vim.cmd 'nnoremap <F10> <cmd>ZenMode<CR>'
         vim.cmd 'inoremap <F10> <cmd>ZenMode<CR>'
      end,
   }
   -- use { -- twilight.nvim:          dim text outside paragraph or function
   --    'folke/twilight.nvim',
   --    config = function()
   --       vim.cmd 'command! TW Twilight'
   --       require("twilight").setup()
   --    end,
   -- }
   use { -- matchup:                   Use the percent (%) key for more things
      'andymass/vim-matchup',
   }
   -- use { -- lightspeed:             Jump around in files quickly
   --    'ggandor/lightspeed.nvim',
   -- }
   use { -- indentline:                Line indent indicators
      'lukas-reineke/indent-blankline.nvim',
      config = function()
         require('blake.other').indent_blankline()
      end
   }
   -- use { -- targets.vim:            fix ci' and other things to work right
   --    'wellle/targets.vim',
   -- }

   -- Conveniences
   use { -- sort motion:               (gs)
      'christoomey/vim-sort-motion',
      config = function()
         vim.cmd [[
            "let g:sort_motion_flags = 'i'
            let g:sort_motion_visual_block_command = 'Vissort'
         ]]
      end,
      requires = 'navicore/vissort.vim', -- config requires
   }
   use { -- ghost:                     Web browser integration
      'raghur/vim-ghost',
      opt = true,
      cmd = {'GhostStart', 'GhostInstall', 'GhostStop', 'GhostSync', 'GhostToggleSync',},
   }
   use { -- undotree:                  visual undo tree
      'mbbill/undotree',
      config = function()
         vim.api.nvim_set_keymap('n', '<Leader>u', '<cmd>UndotreeToggle<CR>', { noremap = true, silent = true, })
      end
   }
  --  use { -- shade.nvim:             dim inactive window
  --    'sunjon/shade.nvim',
  --    config = function()
  --       require'shade'.setup({
  --          overlay_opacity = 50,
  --          opacity_step = 1,
  --          keys = {
  --             brightness_up    = '<C-Up>',
  --             brightness_down  = '<C-Down>',
  --             toggle           = '<Leader>s',
  --          }
  --       })
  --    end
  -- }
   use { -- nvim-autopairs:            automatic quote pairing
      'windwp/nvim-autopairs',
      config = function()
         -- If you want insert `(` after select function or method item
         local cmp_autopairs = require('nvim-autopairs.completion.cmp')
         local cmp = require('cmp')
         cmp.event:on(
            'confirm_done',
            cmp_autopairs.on_confirm_done()
         )
         require('nvim-autopairs').setup({
            check_ts = true,
            map_c_h = true,
            map_c_w = true,
         })
      end,
      requires = 'hrsh7th/nvim-cmp', -- config requires
   }
   use { -- ts-autotag:                automatically close html tags
      'windwp/nvim-ts-autotag',
      config = function()
         require('nvim-ts-autotag').setup()
      end
   }
   use { -- endwise:                   auto add 'end' keyword when typing loops in various languages
   'RRethy/nvim-treesitter-endwise',
   config = function()
      require('nvim-treesitter.configs').setup {
         endwise = {
            enable = true,
         },
      }
   end
}
   use { -- cheat.sh integration
      'dbeniamine/cheat.sh-vim',
   }
   use { -- nvim-align:                Align text
      'RRethy/nvim-align',
   }
   use { -- comment.nvim:              toggle comments - gb and gc
       'numToStr/Comment.nvim',
       config = function()
          require('Comment').setup {
            pre_hook = function(ctx)
               -- Only calculate commentstring for tsx filetypes
               if vim.bo.filetype == 'typescriptreact' then
                  local U = require('Comment.utils')
                  -- Detemine whether to use linewise or blockwise commentstring
                  local type = ctx.ctype == U.ctype.line and '__default' or '__multiline'
                  -- Determine the location where to calculate commentstring from
                  local location = nil
                  if ctx.ctype == U.ctype.block then
                     location = require('ts_context_commentstring.utils').get_cursor_location()
                  elseif ctx.cmotion == U.cmotion.v or ctx.cmotion == U.cmotion.V then
                     location = require('ts_context_commentstring.utils').get_visual_start_location()
                  end
                  return require('ts_context_commentstring.internal').calculate_commentstring({
                     key = type,
                     location = location,
                  })
               end
            end,
          }
       end
    }
   use { -- vim-illuminate:            highlight other occurrences of the word under cursor
      'RRethy/vim-illuminate',
   }
   use { -- filetype.nvim:             detect filetype a lot faster than stock neovim
      'nathom/filetype.nvim',
   }
   use { -- vim-rooter:                cd to the root of a project when opening a file or folder
      'notjedi/nvim-rooter.lua',
   }
   use { -- splitjoin:                 switch between single line and multiline versions of code - gJ and gS
      'AndrewRadev/splitjoin.vim',
   }
   use { -- rhysd: clever-split:       split calculation based on pane dimensions - CS or CleverSplit
      'rhysd/clever-split.vim',
      config = function()
         -- Map :CS to :CleverSplit
         vim.cmd 'command! CS CleverSplit'
      end
   }
   use { -- rhysd: textobj-anyblock:   allow ab or ib instead of i( or a[ or any of the many other block selectors
      'rhysd/vim-textobj-anyblock',
      requires = 'kana/vim-textobj-user',
   }
   -- use { -- rhysd: accelerated jk:  make j and k move faster by pressing them a lot movement
   --    'rhysd/accelerated-jk',
   -- }
   use { -- rhysd: committia:          better commit editing window
      'rhysd/committia.vim',
   }
   use { -- rhysd: conflict-marker:    mark git conflicts - [x, ]x, co, ct, cb, cB, cn
      'rhysd/conflict-marker.vim',
   }
   use { -- lewis6991: spellsitter:    Spell checking in treesitter files
      'lewis6991/spellsitter.nvim',
      config = function()
         require('spellsitter').setup()
      end,
   }
   use { -- lewis6991: spaceless:      Strip trailing whitespace as you are editing
      'lewis6991/spaceless.nvim',
      config = function()
         require'spaceless'.setup()
      end,
   }
   use { -- lewis6991: foldsigns:      signcolumn signs on folded lines
      'lewis6991/foldsigns.nvim',
      config = function()
         require('foldsigns').setup()
      end,
   }
   use { -- lewis6991: cleanfold:      folds have never looked better
      'lewis6991/cleanfold.nvim',
      config = function()
         require('cleanfold').setup()
      end,
   }
   -- use { -- lewis6991: sattelite:   small scroll bar
   --    'lewis6991/satellite.nvim',
   --    config = function()
   --       require('satellite').setup()
   --       vim.cmd [[
   --          nnoremap <C-l> <cmd>nohlsearch\|diffupdate\|SatelliteRefresh<CR>
   --          inoremap <C-l> <cmd>nohlsearch\|diffupdate\|SatelliteRefresh<CR>
   --       ]]
   --    end,
   -- }
   use { -- tpope: surround:           surround text with quotes, parens, tags, and more - ys
      'tpope/vim-surround',
   }
   use { -- tpope: fugitive:           git integration
      'tpope/vim-fugitive',
   }
   use { -- tpope: repeat:             Repeatability for various tpope plugins
      'tpope/vim-repeat',
   }
   use { -- tpope: sleuth:             automatically get info about files and apply it to vim (tabs vs spaces, etc)
      'tpope/vim-sleuth',
   }


   -- Bootstrap packer if needed {
   if (PackerBootstrap) then
      require('packer').sync()
   end
   -- }
end,
config = {
  display = { open_fn = require('packer.util').float },
  -- Move to lua dir so impatient.nvim can cache it
  compile_path = vim.fn.stdpath('config')..'/lua/packer_compiled.lua',
}})

-- vim command to align all plugin descriptions: select all plugins in visual line, :'<,'>Align \(.\+use { -- .\+: \+\)\@<=[^ ][^:]\+$
-- vim:fdm=marker:fmr={,}:fdl=1:expandtab:tabstop=3:sw=3
